@@ -58,10 +58,9 @@ def pytest_configure(config):
         if config.option.base_url:
             status_code = _get_status_code(config.option.base_url)
             assert status_code == 200, 'Base URL did not return status code 200. (URL: %s, Response: %s)' % (config.option.base_url, status_code)
-    
-        report_path = config.option.webqa_report_path
-        if report_path:
-            config._html = LogHTML(report_path, config)
+
+        if config.option.webqa_report_path:
+            config._html = LogHTML(config)
             config.pluginmanager.register(config._html)
 
 
@@ -73,6 +72,7 @@ def pytest_unconfigure(config):
 
 
 def pytest_runtest_setup(item):
+    item.debug = _create_debug()
     item.api = item.config.option.api
     item.host = item.config.option.host
     item.port = item.config.option.port
@@ -85,10 +85,14 @@ def pytest_runtest_setup(item):
     item.browser_version = item.config.option.browser_version
     item.platform = item.config.option.platform
     TestSetup.base_url = item.config.option.base_url
-    TestSetup.timeout = item.config.option.timeout
     TestSetup.default_implicit_wait = 10
     item.sauce_labs_credentials_file = item.config.option.sauce_labs_credentials_file
 
+    if item.config.option.api.upper() == 'RC':
+        TestSetup.timeout = item.config.option.timeout * 1000
+    else:
+        TestSetup.timeout = item.config.option.timeout
+
     if item.sauce_labs_credentials_file:
         item.sauce_labs_credentials = _credentials(item.config.option.sauce_labs_credentials_file)
 
@@ -106,10 +110,25 @@ def pytest_runtest_setup(item):
 
 def pytest_runtest_teardown(item):
     if hasattr(TestSetup, 'selenium') and TestSetup.selenium and not 'skip_selenium' in item.keywords:
-        _capture_debug(item)
         _stop_selenium(item)
 
 
+def pytest_runtest_makereport(__multicall__, item, call):
+    report = __multicall__.execute()
+    if report.when == 'call':
+        report.session_id = getattr(item, 'session_id', None)
+        if hasattr(TestSetup, 'selenium') and TestSetup.selenium and not 'skip_selenium' in item.keywords:
+            if report.skipped and 'xfail' in report.keywords or report.failed and 'xfail' not in report.keywords:
+                _capture_url(item)
+                _capture_screenshot(item)
+                _capture_html(item)
+                _capture_log(item)
+                report.sections.append(('pytest-mozwebqa', _debug_summary(item.debug)))
+            _capture_network_traffic(item)
+            report.debug = item.debug
+    return report
+
+
 def pytest_funcarg__mozwebqa(request):
     return TestSetup(request)
 
@@ -165,6 +184,11 @@ def pytest_addoption(parser):
                      dest='firefox_preferences',
                      metavar='str',
                      help='json string of firefox preferences to set (webdriver).')
+    group._addoption('--chromeopts',
+                     action='store',
+                     dest='chrome_options',
+                     metavar='str',
+                     help='json string of google chrome options to set (webdriver).')
     group._addoption('--browser',
                      action='store',
                      dest='browser',
@@ -192,9 +216,9 @@ def pytest_addoption(parser):
     group._addoption('--timeout',
                      action='store',
                      type='int',
-                     default=60000,
+                     default=60,
                      metavar='num',
-                     help='timeout for page loads, etc (selenium rc).')
+                     help='timeout (in seconds) for page loads, etc.')
     group._addoption('--capturenetwork',
                      action='store_true',
                      dest='capture_network',
@@ -239,6 +263,22 @@ def _get_status_code(url):
         print 'Unable to connect to: %s' % url
 
 
+def _create_debug():
+    return {
+        'urls': [],
+        'screenshots': [],
+        'html': [],
+        'logs': [],
+        'network_traffic': []}
+
+
+def _debug_summary(debug):
+    summary = []
+    if debug['urls']:
+        summary.append('Failing URL: %s' % debug['urls'][-1])
+    return '\n'.join(summary)
+
+
 def _credentials(credentials_file):
     stream = file(credentials_file, 'r')
     credentials = yaml.load(stream)
@@ -270,7 +310,7 @@ def _check_selenium_usage(item):
     '''
         Check that the usage parameters are correct. If wrong throws the appropriate error
     '''
-    if TestSetup.base_url is None:
+    if not TestSetup.base_url:
         raise pytest.UsageError('--baseurl must be specified.')
 
     if item.sauce_labs_credentials_file:
@@ -299,8 +339,8 @@ def _get_common_sauce_settings(item):
     return {'build': item.config.option.build or None,
             'name': '.'.join(_split_class_and_test_names(item.nodeid)),
             'tags': tags,
-            'public': True,
-            'restricted-public-info': True}
+            'public': 'private' not in item.keywords,
+            'restricted-public-info': 'public' not in item.keywords}
 
 
 def _create_firefox_profile(preferences):
@@ -315,11 +355,31 @@ def _create_firefox_profile(preferences):
     else:
         return None
 
+
+def _create_chrome_options(preferences):
+    options = webdriver.ChromeOptions()
+    options_from_json = json.loads(preferences)
+
+    if 'arguments' in options_from_json:
+        for args_ in options_from_json['arguments']:
+            options.add_argument(args_)
+
+    if 'extensions' in options_from_json:
+        for ext_ in options_from_json['extensions']:
+            options.add_extension(ext_)
+
+    if 'binary_location' in options_from_json:
+        options.binary_location = options_from_json['binary_location']
+
+    return options
+
+
 def _start_selenium(item):
     if item.api == 'webdriver':
         _start_webdriver_client(item)
     else:
         _start_rc_client(item)
+    _capture_session_id(item)
 
 
 def _start_webdriver_client(item):
@@ -331,11 +391,13 @@ def _start_webdriver_client(item):
         executor = 'http://%s:%s@ondemand.saucelabs.com:80/wd/hub' % (item.sauce_labs_credentials['username'], item.sauce_labs_credentials['api-key'])
         TestSetup.selenium = webdriver.Remote(command_executor=executor,
                                               desired_capabilities=capabilities)
-        _capture_session_id(item, _debug_path(item))
     else:
         profile = _create_firefox_profile(item.config.option.firefox_preferences)
         if item.driver.upper() == 'REMOTE':
-            capabilities = getattr(webdriver.DesiredCapabilities, item.browser_name.upper())
+            if item.config.option.chrome_options:
+                capabilities = _create_chrome_options(item.config.option.chrome_options).to_capabilities()
+            else:
+                capabilities = getattr(webdriver.DesiredCapabilities, item.browser_name.upper())
             capabilities['version'] = item.browser_version
             capabilities['platform'] = item.platform.upper()
             executor = 'http://%s:%s/wd/hub' % (item.host, item.port)
@@ -348,10 +410,19 @@ def _start_webdriver_client(item):
                 raise AttributeError("Invalid browser name: '%s'. Valid options are: %s" % (item.browser_name, ', '.join(valid_browsers)))
 
         elif item.driver.upper() == 'CHROME':
-            if hasattr(item, 'chrome_path'):
-                TestSetup.selenium = webdriver.Chrome(executable_path=item.chrome_path)
+            if item.config.option.chrome_path:
+                if item.config.option.chrome_options:
+                    options = _create_chrome_options(item.config.option.chrome_options)
+                    TestSetup.selenium = webdriver.Chrome(executable_path=item.chrome_path,
+                                                          chrome_options=options)
+                else:
+                    TestSetup.selenium = webdriver.Chrome(executable_path=item.chrome_path)
             else:
-                TestSetup.selenium = webdriver.Chrome()
+                if item.config.option.chrome_options:
+                    options = _create_chrome_options(item.config.option.chrome_options)
+                    TestSetup.selenium = webdriver.Chrome(chrome_options=options)
+                else:
+                    TestSetup.selenium = webdriver.Chrome()
 
         elif item.driver.upper() == 'FIREFOX':
             binary = hasattr(item, 'firefox_path') and FirefoxBinary(item.firefox_path) or None
@@ -386,37 +457,10 @@ def _start_rc_client(item):
     else:
         TestSetup.selenium.start()
 
-    if item.sauce_labs_credentials_file:
-        _capture_session_id(item, _debug_path(item))
-
     TestSetup.selenium.set_timeout(TestSetup.timeout)
     TestSetup.selenium.set_context(".".join(_split_class_and_test_names(item.nodeid)))
 
 
-def _debug_path(item):
-    report_path = item.config.option.webqa_report_path
-    debug_path = report_path and \
-                 os.path.dirname(report_path) and \
-                 os.path.sep.join([os.path.dirname(report_path), 'debug']) or 'debug'
-    debug_path = os.path.normpath(os.path.expanduser(os.path.expandvars(debug_path)))
-    if not os.path.exists(debug_path):
-        os.makedirs(debug_path)
-    return os.path.sep.join([debug_path, _generate_filename(*_split_class_and_test_names(item.nodeid))])
-
-
-def _capture_debug(item):
-    filename = _debug_path(item)
-    _capture_screenshot(item, filename)
-    _capture_html(item, filename)
-    #_capture_log(item, filename)  # Log may contain passwords, etc so we shouldn't capture this until we can do so safely
-    if item.config.option.capture_network:
-        _capture_network(filename)
-
-
-def _generate_filename(classname, testname):
-    return '%s_%s' % (classname.replace('.', '_'), testname)
-
-
 def _split_class_and_test_names(nodeid):
     names = nodeid.split("::")
     names[0] = names[0].replace("/", '.')
@@ -428,47 +472,48 @@ def _split_class_and_test_names(nodeid):
     return (classname, name)
 
 
-def _capture_session_id(item, filename):
-    f = open("%s.session" % filename, 'wb')
+def _capture_session_id(item):
     if item.api.upper() == 'WEBDRIVER':
-        f.write(TestSetup.selenium.session_id)
+        session_id = TestSetup.selenium.session_id
     else:
-        f.write(TestSetup.selenium.get_eval('selenium.sessionId'))
-    f.close()
+        session_id = TestSetup.selenium.get_eval('selenium.sessionId')
+    item.session_id = session_id
 
 
-def _capture_screenshot(item, filename):
-    try:
-        f = open("%s.png" % filename, 'wb')
-        if item.api.upper() == 'WEBDRIVER':
-            f.write(base64.decodestring(TestSetup.selenium.get_screenshot_as_base64()))
-        else:
-            f.write(base64.decodestring(TestSetup.selenium.capture_entire_page_screenshot_to_string('')))
-        f.close()
-    except:
-        pass
-
-
-def _capture_html(item, filename):
-    f = open("%s.html" % filename, 'wb')
+def _capture_screenshot(item):
     if item.api.upper() == 'WEBDRIVER':
-        f.write(TestSetup.selenium.page_source.encode('utf-8'))
+        screenshot = TestSetup.selenium.get_screenshot_as_base64()
     else:
-        f.write(TestSetup.selenium.get_html_source().encode('utf-8'))
-    f.close()
+        screenshot = TestSetup.selenium.capture_entire_page_screenshot_to_string('')
+    item.debug['screenshots'].append(screenshot)
 
 
-def _capture_log(item, filename):
+def _capture_html(item):
+    if item.api.upper() == 'WEBDRIVER':
+        html = TestSetup.selenium.page_source.encode('utf-8')
+    else:
+        html = TestSetup.selenium.get_html_source().encode('utf-8')
+    item.debug['html'].append(html)
+
+
+def _capture_log(item):
     if item.api.upper() == 'RC':
-        f = open("%s.log" % filename, 'wb')
-        f.write(TestSetup.selenium.get_log().encode('utf-8'))
-        f.close()
+        log = TestSetup.selenium.get_log().encode('utf-8')
+        item.debug['logs'].append(log)
 
 
-def _capture_network(filename):
-    f = open('%s.json' % filename, 'w')
-    f.write(TestSetup.selenium.captureNetworkTraffic('json'))
-    f.close()
+def _capture_network_traffic(item):
+    if item.api.upper() == 'RC' and item.config.option.capture_network:
+        network_traffic = TestSetup.selenium.captureNetworkTraffic('json')
+        item.debug['network_traffic'].append(network_traffic)
+
+
+def _capture_url(item):
+    if item.api.upper() == 'WEBDRIVER':
+        url = TestSetup.selenium.current_url
+    else:
+        url = TestSetup.selenium.get_location()
+    item.debug['urls'].append(url)
 
 
 def _stop_selenium(item):
@@ -486,9 +531,10 @@ def _stop_selenium(item):
 
 class LogHTML(object):
 
-    def __init__(self, logfile, config):
-        logfile = os.path.expanduser(os.path.expandvars(logfile))
+    def __init__(self, config):
+        logfile = os.path.expanduser(os.path.expandvars(config.option.webqa_report_path))
         self.logfile = os.path.normpath(logfile)
+        self._debug_path = 'debug'
         self.config = config
         self.test_logs = []
         self.errors = self.errors = 0
@@ -496,21 +542,58 @@ class LogHTML(object):
         self.failed = self.failed = 0
         self.xfailed = self.xpassed = 0
 
+    def _debug_paths(self, testclass, testmethod):
+        root_path = os.path.join(os.path.dirname(self.logfile), self._debug_path)
+        root_path = os.path.normpath(os.path.expanduser(os.path.expandvars(root_path)))
+        test_path = os.path.join(testclass.replace('.', '_'), testmethod)
+        full_path = os.path.join(root_path, test_path)
+        if not os.path.exists(full_path):
+            os.makedirs(full_path)
+        relative_path = os.path.join(self._debug_path, test_path)
+        absolute_path = os.path.join(root_path, test_path)
+        return (relative_path, full_path)
+
     def _appendrow(self, result, report):
-        (classname, testname) = _split_class_and_test_names(report.nodeid)
+        (testclass, testmethod) = _split_class_and_test_names(report.nodeid)
         time = getattr(report, 'duration', 0.0)
-        session_id = self._get_session_id(report)
-        links = {'HTML': self._get_debug_filename(report, 'html'),
-                 'Screenshot': self._get_debug_filename(report, 'png')}
-        if self.config.option.capture_network:
-            links['Network'] = self._get_debug_filename(report, 'json')
-        if session_id:
-            links['Sauce Labs Job'] = 'http://saucelabs.com/jobs/%s' % session_id
-        # Log may contain passwords, etc so we shouldn't capture this until we can do so safely
-        #if self.config.option.api.upper() == 'RC':
-        #    links['Log'] = '%s.log' % filename
+
+        links = {}
+        if hasattr(report, 'debug') and any(report.debug.values()):
+            (relative_path, full_path) = self._debug_paths(testclass, testmethod)
+
+            if report.debug['screenshots']:
+                filename = 'screenshot.png'
+                f = open(os.path.join(full_path, filename), 'wb')
+                f.write(base64.decodestring(report.debug['screenshots'][-1]))
+                links.update({'Screenshot': os.path.join(relative_path, filename)})
+
+            if report.debug['html']:
+                filename = 'html.txt'
+                f = open(os.path.join(full_path, filename), 'wb')
+                f.write(report.debug['html'][-1])
+                links.update({'HTML': os.path.join(relative_path, filename)})
+
+            # Log may contain passwords, etc so we only capture it for tests marked as public
+            if report.debug['logs'] and 'public' in report.keywords:
+                filename = 'log.txt'
+                f = open(os.path.join(full_path, filename), 'wb')
+                f.write(report.debug['logs'][-1])
+                links.update({'Log': os.path.join(relative_path, filename)})
+
+            if report.debug['network_traffic']:
+                filename = 'networktraffic.json'
+                f = open(os.path.join(full_path, filename), 'wb')
+                f.write(report.debug['network_traffic'][-1])
+                links.update({'Network Traffic': os.path.join(relative_path, filename)})
+
+            if report.debug['urls']:
+                links.update({'Failing URL': report.debug['urls'][-1]})
+
+        if self.config.option.sauce_labs_credentials_file and hasattr(report, 'session_id'):
+            links['Sauce Labs Job'] = 'http://saucelabs.com/jobs/%s' % report.session_id
+
         links_html = []
-        self.test_logs.append('\n<tr class="%s"><td class="%s">%s</td><td>%s</td><td>%s</td><td>%is</td>' % (result.lower(), result.lower(), result, classname, testname, round(time)))
+        self.test_logs.append('<tr class="%s"><td class="%s">%s</td><td>%s</td><td>%s</td><td>%is</td>' % (result.lower(), result.lower(), result, testclass, testmethod, round(time)))
         self.test_logs.append('<td>')
         for name, path in links.iteritems():
             links_html.append('<a href="%s">%s</a>' % (path, name))
@@ -518,10 +601,10 @@ class LogHTML(object):
         self.test_logs.append('</td>')
 
         if not 'Passed' in result:
-            self.test_logs.append('\n<tr class="additional"><td colspan="5">')
+            self.test_logs.append('<tr class="additional"><td colspan="5">')
 
             if report.longrepr:
-                self.test_logs.append('\n<div class="log">')
+                self.test_logs.append('<div class="log">')
                 for line in str(report.longrepr).splitlines():
                     separator = line.startswith('_ ' * 10)
                     if separator:
@@ -533,17 +616,19 @@ class LogHTML(object):
                         else:
                             self.test_logs.append(cgi.escape(line))
                     self.test_logs.append('<br />')
-                self.test_logs.append('\n</div>')
+                self.test_logs.append('</div>')
 
-            self.test_logs.append('\n<div class="screenshot"><a href="%s"><img src="%s" /></a></div>' % (links['Screenshot'], links['Screenshot']))
-            if session_id:
-                self.test_logs.append('\n<div id="player%s" class="video">' % session_id)
-                self.test_logs.append('\n<object width="100%" height="100%" type="application/x-shockwave-flash" data="http://saucelabs.com/flowplayer/flowplayer-3.2.5.swf?0.2566397726976729" name="player_api" id="player_api">')
-                self.test_logs.append('\n<param value="true" name="allowfullscreen">')
-                self.test_logs.append('\n<param value="always" name="allowscriptaccess">')
-                self.test_logs.append('\n<param value="high" name="quality">')
-                self.test_logs.append('\n<param value="true" name="cachebusting">')
-                self.test_logs.append('\n<param value="#000000" name="bgcolor">')
+            if 'Screenshot' in links:
+                self.test_logs.append('<div class="screenshot"><a href="%s"><img src="%s" /></a></div>' % (links['Screenshot'], links['Screenshot']))
+
+            if self.config.option.sauce_labs_credentials_file and hasattr(report, 'session_id'):
+                self.test_logs.append('<div id="player%s" class="video">' % report.session_id)
+                self.test_logs.append('<object width="100%" height="100%" type="application/x-shockwave-flash" data="http://saucelabs.com/flowplayer/flowplayer-3.2.5.swf?0.2566397726976729" name="player_api" id="player_api">')
+                self.test_logs.append('<param value="true" name="allowfullscreen">')
+                self.test_logs.append('<param value="always" name="allowscriptaccess">')
+                self.test_logs.append('<param value="high" name="quality">')
+                self.test_logs.append('<param value="true" name="cachebusting">')
+                self.test_logs.append('<param value="#000000" name="bgcolor">')
                 flash_vars = 'config={\
                     &quot;clip&quot;:{\
                         &quot;url&quot;:&quot;http%%3A//saucelabs.com/jobs/%s/video.flv&quot;,\
@@ -562,22 +647,10 @@ class LogHTML(object):
                         &quot;url&quot;:&quot;http%%3A//saucelabs.com/jobs/%s/video.flv&quot;,\
                         &quot;provider&quot;:&quot;streamer&quot;,\
                         &quot;autoPlay&quot;:false,\
-                        &quot;autoBuffering&quot;:true}]}' % (session_id, session_id, session_id)
-                self.test_logs.append('\n<param value="%s" name="flashvars">' % flash_vars.replace(' ', ''))
-                self.test_logs.append('\n</object></div>')
-            self.test_logs.append('\n</td></tr>')
-
-    def _get_debug_filename(self, report, extension):
-        filename = os.path.join('debug', _generate_filename(*_split_class_and_test_names(report.nodeid)))
-        return '%s.%s' % (filename, extension)
-
-    def _get_session_id(self, report):
-        report_dir = self._make_report_dir()
-        try:
-            session_id = open(os.path.join(report_dir, self._get_debug_filename(report, 'session')), 'r').readline()
-        except:
-            session_id = None
-        return session_id
+                        &quot;autoBuffering&quot;:true}]}' % (report.session_id, report.session_id, report.session_id)
+                self.test_logs.append('<param value="%s" name="flashvars">' % flash_vars.replace(' ', ''))
+                self.test_logs.append('</object></div>')
+            self.test_logs.append('</td></tr>')
 
     def _make_report_dir(self):
         logfile_dirname = os.path.dirname(self.logfile)
@@ -586,19 +659,16 @@ class LogHTML(object):
         return logfile_dirname
 
     def _send_result_to_sauce(self, report):
-        session_id = self._get_session_id(report)
-
-        if session_id:
+        if hasattr(report, 'session_id'):
             try:
                 result = {'passed': report.passed or (report.failed and 'xfail' in report.keywords)}
                 credentials = _credentials(self.config.option.sauce_labs_credentials_file)
                 basic_authentication = ('%s:%s' % (credentials['username'], credentials['api-key'])).encode('base64')[:-1]
                 connection = httplib.HTTPConnection('saucelabs.com')
-                connection.request('PUT', '/rest/v1/%s/jobs/%s' % (credentials['username'], session_id),
+                connection.request('PUT', '/rest/v1/%s/jobs/%s' % (credentials['username'], report.session_id),
                                    json.dumps(result),
-                                   headers = {
-                                       'Authorization': 'Basic %s' % basic_authentication,
-                                       'Content-Type': 'text/json'})
+                                   headers={'Authorization': 'Basic %s' % basic_authentication,
+                                            'Content-Type': 'text/json'})
                 connection.getresponse()
             except:
                 pass
@@ -632,7 +702,8 @@ class LogHTML(object):
             self._send_result_to_sauce(report)
 
         if report.passed:
-            self.append_pass(report)
+            if report.when == 'call':
+                self.append_pass(report)
         elif report.failed:
             if report.when != "call":
                 self.append_error(report)
@@ -651,63 +722,69 @@ class LogHTML(object):
         suite_stop_time = time.time()
         suite_time_delta = suite_stop_time - self.suite_start_time
         numtests = self.passed + self.failed + self.xpassed + self.xfailed
-        logfile.write('<html><head><title>Test Report</title><style>')
-        logfile.write('\nbody {font-family: Helvetica, Arial, sans-serif; font-size: 12px}')
-        logfile.write('\na {color: #999}')
-        logfile.write('\nh2 {font-size: 16px}')
-        logfile.write('\ntable {border: 1px solid #e6e6e6; color: #999; font-size: 12px; border-collapse: collapse}')
-        logfile.write('\n#configuration tr:nth-child(odd) {background-color: #f6f6f6}')
-        logfile.write('\nth, td {padding: 5px; border: 1px solid #E6E6E6; text-align: left}')
-        logfile.write('\nth {font-weight: bold}')
-        logfile.write('\ntr.passed, tr.skipped, tr.xfailed, tr.error, tr.failed, tr.xpassed {color: inherit}')
-        logfile.write('\ntr.passed + tr.additional {display: none}')
-        logfile.write('\n.passed {color: green}')
-        logfile.write('\n.skipped, .xfailed {color: orange}')
-        logfile.write('\n.error, .failed, .xpassed {color: red}')
-        logfile.write('\n.log {display: inline-block; width: 800px; height: 230px; overflow-y: scroll; color: black; border: 1px solid #E6E6E6; padding: 5px; background-color: #E6E6E6; font-family: "Courier New", Courier, monospace; white-space: pre}')
-        logfile.write('\n.screenshot {display: inline-block; border: 1px solid #E6E6E6; width: 320px; height: 240px; overflow: hidden}')
-        logfile.write('\n.screenshot img {width: 320px}')
-        logfile.write('\n.video {display: inline-block; width: 320px; height: 240px}')
-        logfile.write('\n</style></head><body>')
-        logfile.write('\n<h2>Configuration</h2>')
-        logfile.write('\n<table id="configuration"><tr><th>Base URL</th><td><a href="%s">%s</td></tr>' % (self.config.option.base_url, self.config.option.base_url))
-        if self.config.option.build:
-            logfile.write('\n<tr><th>Build</th><td>%s</td></tr>' % self.config.option.build)
-        logfile.write('\n<tr><th>Selenium API</th><td>%s</td></tr>' % self.config.option.api)
-        if not self.config.option.driver.upper() == 'REMOTE':
-            logfile.write('\n<tr><th>Driver</th><td>%s</td></tr>' % self.config.option.driver)
-            if self.config.option.firefox_path:
-                logfile.write('\n<tr><th>Firefox Path</th><td>%s</td></tr>' % self.config.option.firefox_path)
-            elif self.config.option.chrome_path:
-                logfile.write('\n<tr><th>Google Chrome Path</th><td>%s</td></tr>' % self.config.option.chrome_path)
-        else:
-            selenium_server = self.config.option.sauce_labs_credentials_file and 'Sauce Labs' or 'http://%s:%s' % (self.config.option.host, self.config.option.port)
-            logfile.write('\n<tr><th>Selenium Server</th><td>%s</td></tr>' % selenium_server)
-            if self.config.option.api.upper() == 'WEBDRIVER' or self.config.option.sauce_labs_credentials_file:
-                logfile.write('\n<tr><th>Browser</th><td>%s %s on %s</td></tr>' % (str(self.config.option.browser_name).title(), self.config.option.browser_version, str(self.config.option.platform).title()))
-            else:
-                logfile.write('\n<tr><th>Browser</th><td>%s</td></tr>' % self.config.option.environment or self.config.option.browser)
-            if self.config.option.api.upper() == 'RC':
-                logfile.write('\n<tr><th>Timeout</th><td>%s</td></tr>' % self.config.option.timeout)
-        logfile.write('\n<tr><th>Capture Network Traffic</th><td>%s</td></tr>' % self.config.option.capture_network)
-        if self.config.option.credentials_file:
-            logfile.write('\n<tr><th>Credentials</th><td>%s</td></tr>' % self.config.option.credentials_file)
-        if self.config.option.sauce_labs_credentials_file:
-            logfile.write('\n<tr><th>Sauce Labs Credentials</th><td>%s</td></tr>' % self.config.option.sauce_labs_credentials_file)
-        logfile.write('\n</table>')
-        logfile.write('\n<h2>Summary</h2>')
-        logfile.write('\n<p>%i tests ran in %i seconds.<br />' % (numtests, suite_time_delta))
-        logfile.write('\n<span class="passed">%i passed</span>, ' % self.passed)
-        logfile.write('<span class="skipped">%i skipped</span>, ' % self.skipped)
-        logfile.write('<span class="failed">%i failed</span>, ' % self.failed)
-        logfile.write('<span class="error">%i errors</span>.<br />' % self.errors)
-        logfile.write('\n<span class="skipped">%i expected failures</span>, ' % self.xfailed)
-        logfile.write('<span class="failed">%i unexpected passes</span>.</p>' % self.xpassed)
-        logfile.write('\n<h2>Results</h2>')
-        logfile.write('\n<table id="results">')
-        logfile.write('\n<tr><th>Result</th><th>Class</th><th>Name</th><th>Duration</th><th>Links</th></tr>')
-        logfile.writelines(self.test_logs)
-        logfile.write('\n</table></body></html>')
+
+        server = self.config.option.sauce_labs_credentials_file and \
+                 'Sauce Labs' or 'http://%s:%s' % (self.config.option.host, self.config.option.port)
+        browser = self.config.option.browser_name and \
+                  self.config.option.browser_version and \
+                  self.config.option.platform and \
+                  '%s %s on %s' % (str(self.config.option.browser_name).title(),
+                                   self.config.option.browser_version,
+                                   str(self.config.option.platform).title()) or \
+                  self.config.option.environment or \
+                  self.config.option.browser
+
+        configuration = {
+            'Base URL': self.config.option.base_url,
+            'Build': self.config.option.build,
+            'Selenium API': self.config.option.api,
+            'Driver': self.config.option.driver,
+            'Firefox Path': self.config.option.firefox_path,
+            'Google Chrome Path': self.config.option.chrome_path,
+            'Selenium Server': server,
+            'Browser': browser,
+            'Timeout': self.config.option.timeout,
+            'Capture Network Traffic': self.config.option.capture_network,
+            'Credentials': self.config.option.credentials_file,
+            'Sauce Labs Credentials': self.config.option.sauce_labs_credentials_file}
+
+        html = []
+        html.append('<html><head><title>Test Report</title><style>')
+        html.append('body {font-family: Helvetica, Arial, sans-serif; font-size: 12px}')
+        html.append('a {color: #999}')
+        html.append('h2 {font-size: 16px}')
+        html.append('table {border: 1px solid #e6e6e6; color: #999; font-size: 12px; border-collapse: collapse}')
+        html.append('#configuration tr:nth-child(odd) {background-color: #f6f6f6}')
+        html.append('th, td {padding: 5px; border: 1px solid #E6E6E6; text-align: left}')
+        html.append('th {font-weight: bold}')
+        html.append('tr.passed, tr.skipped, tr.xfailed, tr.error, tr.failed, tr.xpassed {color: inherit}')
+        html.append('tr.passed + tr.additional {display: none}')
+        html.append('.passed {color: green}')
+        html.append('.skipped, .xfailed {color: orange}')
+        html.append('.error, .failed, .xpassed {color: red}')
+        html.append('.log {display: inline-block; width: 800px; height: 230px; overflow-y: scroll; color: black; border: 1px solid #E6E6E6; padding: 5px; background-color: #E6E6E6; font-family: "Courier New", Courier, monospace; white-space: pre}')
+        html.append('.screenshot {display: inline-block; border: 1px solid #E6E6E6; width: 320px; height: 240px; overflow: hidden}')
+        html.append('.screenshot img {width: 320px}')
+        html.append('.video {display: inline-block; width: 320px; height: 240px}')
+        html.append('</style></head><body>')
+        html.append('<h2>Configuration</h2>')
+        html.append('<table id="configuration">')
+        html.append('\n'.join(['<tr><td>%s</td><td>%s</td></tr>' % (k, v) for k, v in sorted(configuration.items()) if v]))
+        html.append('</table>')
+        html.append('<h2>Summary</h2>')
+        html.append('<p>%i tests ran in %i seconds.<br />' % (numtests, suite_time_delta))
+        html.append('<span class="passed">%i passed</span>, ' % self.passed)
+        html.append('<span class="skipped">%i skipped</span>, ' % self.skipped)
+        html.append('<span class="failed">%i failed</span>, ' % self.failed)
+        html.append('<span class="error">%i errors</span>.<br />' % self.errors)
+        html.append('<span class="skipped">%i expected failures</span>, ' % self.xfailed)
+        html.append('<span class="failed">%i unexpected passes</span>.</p>' % self.xpassed)
+        html.append('<h2>Results</h2>')
+        html.append('<table id="results">')
+        html.append('<tr><th>Result</th><th>Class</th><th>Name</th><th>Duration</th><th>Links</th></tr>')
+        html.append(''.join(self.test_logs))
+        html.append('</table></body></html>')
+        logfile.write('\n'.join(html))
         logfile.close()
 
 
